/*
 * Unitex
 *
 * Copyright (C) 2001-2012 Université Paris-Est Marne-la-Vallée <unitex@univ-mlv.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.umlv.unitex.process.commands;

import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import fr.umlv.unitex.console.Console;
import fr.umlv.unitex.console.ConsoleEntry;
import fr.umlv.unitex.console.Couple;
import fr.umlv.unitex.process.ExecParameters;

/**
 * @author Sébastien Paumier
 */
public class ErrorMessageCommand extends CommandBuilder {
	private final String message;

	public ErrorMessageCommand(String mess) {
		super("");
		type = ERROR_MESSAGE;
		message = mess;
	}

	public String getMessage() {
		return message;
	}

	@Override
	public ConsoleEntry logIntoConsole() {
		return Console.addCommand(
				"Error message emitted by the graphical interface", true, null);
	}

	@Override
	public boolean executeCommand(final ExecParameters p,
			final ConsoleEntry entry) {
		if (p.getStderr() == null)
			return true;
		final ErrorMessageCommand c = this;
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					p.getStderr().addLine(new Couple(c.getMessage(), true));
				}
			});
		} catch (final InterruptedException e) {
			return false;
		} catch (final InvocationTargetException e) {
			return false;
		}
		return true;
	}
}
